
//#include <avr/wdt.h>
#include <avr/io.h>             // standard input/output
#include <avr/interrupt.h>      // for sei()
#include <util/delay.h>         // for _delay_ms()
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "osccal.h"
#include "./usbdrv/usbdrv.h"    // usbdriver by http://www.obdev.at

#define UTIL_BIN4(x)        ((uchar)((0##(x) & 01000)/64 + (0##(x) & 0100)/16 + (0##(x) & 010)/4 + (0##(x) & 1)))
#define UTIL_BIN8(hi, lo)   ((uchar)(UTIL_BIN4(hi) * 16 + UTIL_BIN4(lo)))

// PC1 -> Rec
// PC0 -> BTN (resistively multiplexed pin for two buttons)
// PB3 -> IR

/* -------------------------------------------------------------------------- */
/* ------------------------------ USB interface ----------------------------- */
/* -------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[22] = {    /* USB report descriptor */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0xfe,                    //   REPORT_COUNT (254)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};
/* Since we define only one feature report, we don't use report-IDs (which
 * would be the first byte of the report). The entire report consists of 245
 * opaque data bytes.
 */

#define RESOLUTION   20     ///< Time duration in milli seconds for shortest IR pulse
#define MAXPULSE   6000     ///< Maximal length for an IR pulse in units of ::RESOLUTION
#define BSIZE       127     ///< Number of halve wave pulse that can be stored
#define BTN_RPL      20     ///< Minimum ADC readout for replay button
#define BTN_REC     128     ///< Minimum ADC readout for record button
uint8_t pulses8[2*BSIZE];   ///< Array for usb data transfer. Stores timing for the IR pulses (in units of a number of ms equal to ::RESOLUTION)
uint16_t* pulses = (uint16_t*)pulses8; ///< Conveninace pointer for 16 bit pulse lengths
uint8_t currentpulse = 0;   ///< Current postion in ::pulses
uint8_t currentpulse8= 0;

/* -------------------------------------------------------------------------- */

// Send Data
usbMsgLen_t usbFunctionSetup(uchar data[8]) {
    usbRequest_t *rq = (void *)data;

    if( (rq->bmRequestType & USBRQ_TYPE_MASK)==USBRQ_TYPE_CLASS ) {
        if(rq->bRequest == USBRQ_HID_GET_REPORT){
            usbMsgPtr = pulses8;
            return (2*BSIZE);
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            currentpulse = 0;   currentpulse8 = 0;
            return USB_NO_MSG;
        }
    }
    return 0;
}

// Receive Data
uchar usbFunctionWrite(uchar *data, uchar len) {
    uint8_t i;
    if(len > (2*BSIZE)-currentpulse8)
        len = (2*BSIZE)-currentpulse8;
    for(i = 0; i<len; i++)
        pulses8[currentpulse8++] = data[i];
    currentpulse = (currentpulse8>>1);
    return (2*BSIZE)-currentpulse8-len;
}

/* -------------------------------------------------------------------------- */

void record_pattern(void) {
    uint8_t currentStatus = PINC & (1<<PC1);
    uint8_t lastStatus = currentStatus;
    pulses[BSIZE-1] = RESOLUTION;

    while( ADCH > BTN_RPL ); // wait for button to be released

    while( currentStatus == lastStatus ) { // wait for pin to go low (counter-intuitively that means the IR receiver gets an IR signal)
        currentStatus = (PINC & (1<<PC1));
        if( ADCH > BTN_RPL ) {          // If no ir signal has arrived and a button is pressend+
            while( ADCH > BTN_RPL );    //+wait for button release+
            return;                     //+then abort recording
        }
    }
    lastStatus = currentStatus;

    currentpulse = 0;
    pulses[BSIZE-2] = 0xaaaa;   // code: IR signal received
    while( currentpulse<BSIZE-2 ) {
        pulses[currentpulse] = 0;
        while( currentStatus == lastStatus ) {
            pulses[currentpulse]++;
            if ((pulses[currentpulse] >= MAXPULSE) && (currentpulse > 1)) {
                pulses[currentpulse+1] = 0xbbbb; // code: normal termination
                return;
            }
            _delay_us(RESOLUTION);
            currentStatus = (PINC & (1<<PC1));
        }
        lastStatus = currentStatus;
        currentpulse++;
    }
    pulses[BSIZE-2] = 0xdddd; // code: out of memory
}

// Sends a 38kHz modulated pulse to the IRledPin for a certain # of microseconds
void pulseIR(uint32_t pulselength) {
    pulselength *= RESOLUTION; // conversion to microseconds
    cli();  // this turns off any background interrupts
    while (pulselength > 25) {
        pulselength -= 26; // 38 kHz is about 26 microseconds
        // Here we generate the wave:
        PORTB |=  (1<<PB3); // Takes about 3 microseconds to happen
        _delay_us(13);
        PORTB &=~ (1<<PB3);
        _delay_us(13);
    }
    sei();  // this turns them back on
}

void repaly_pattern(void) {
    uint8_t i;
    uint16_t j;
    for(i=0; i<currentpulse-1; i+=2) { // we have 16bit types -> hence division by 2 and handle tow pulses (one HIGH, one LOW) in a single iteration (hence the +=2)
        pulseIR(   pulses[ i ] );          // HIGH pulse
        for(j=0; j<pulses[i+1]; ++j) {
            _delay_us( RESOLUTION );       // LOW pulse
        }
    }
}

/* -------------------------------------------------------------------------- */

int main(void) {
    // SETUP
    DDRB |= (1<<PB3);                   // Pin connected to led is output
    DDRC &=~(1<<PC1);                   // Pin connected to ir receiver is intput (debug)
    ADCSRB &=~( (1<<ADTS2) | (1<<ADTS1) | (1<<ADTS0) );        // free running mode for continuous samples
    ADMUX = (1<<REFS0) | (1<<ADLAR);    // reference Vcc, 8bit precision, source P0
    ADCSRA = (1<<ADEN) | (1<<ADATE) | (1<<ADPS2) | (1<<ADPS1); // enable, auto-trigger, 1/64 prescaler
    ADCSRA |= (1<<ADSC);                // start converting

    usbInit();
    usbDeviceDisconnect(); _delay_ms(250); usbDeviceConnect(); // force reenumberation
    sei();

    // DO THE LOOPING
    pulses[BSIZE-1] = RESOLUTION;
    while(1) {
        usbPoll();
        if( pulses[BSIZE-1] == 0x0000 ) {
            pulses[BSIZE-1] = RESOLUTION;
            repaly_pattern();
        } else if( pulses[0] == 0 && pulses[1] == 0x1111 ) {
            currentpulse = 0;
            record_pattern();
        }
        if        ( ADCH > BTN_REC ) {   // record button pressed
            record_pattern();
        } else if ( ADCH > BTN_RPL ) {   // replay button pressed
            repaly_pattern();
        }
    }
    return 0;
}

/* -------------------------------------------------------------------------- */
